% Locally c-optimal for Example 6 (Magnusdottir,2013)
% Two-model y1=emax*x/ed50+x; y2=smax*x/sd50+x;
% Express as y1=a1*x/b1+x; y2=a2*x/b2+x;
% CUI=k1*y1-k2*y2
% g=max E(CUI)=((k1*k2*b1*a1*b2*a2)^(1/2)*(b1-b2)-b1*b2*(k1*a1-k2*a2))/(k1*b1*a1-k2*b2*a2);
% design space is [0,500]
% CVX
clear;
N=10001; % number of candidate points in design space

% parameter value
k1=1;
k2=1;
alpha1=1; %emax
alpha2=1; %smax
beta1=1;  %ed50
beta2=2;  %sd50

syms a1 b1 a2 b2;
g=((k1*k2*b1*a1*b2*a2)^(1/2)*(b1-b2)-b1*b2*(k1*a1-k2*a2))/(k1*b1*a1-k2*b2*a2);
ga1=diff(g,a1); %partial derivative with respect to emax
gb1=diff(g,b1); %partial derivative with respect to ed50
ga2=diff(g,a2); %partial derivative with respect to smax
gb2=diff(g,b2); %partial derivative with respect to sd50

my_ga1 = subs(ga1,{a1,b1,a2,b2},{alpha1,beta1,alpha2,beta2}); %sub values 
my_gb1 = subs(gb1,{a1,b1,a2,b2},{alpha1,beta1,alpha2,beta2});  
my_ga2 = subs(ga2,{a1,b1,a2,b2},{alpha1,beta1,alpha2,beta2});  
my_gb2 = subs(gb2,{a1,b1,a2,b2},{alpha1,beta1,alpha2,beta2});

my_g=[my_gb1 my_ga1 my_gb2 my_ga2]; %c vector in c-optimal

% sigma0
r12=0; %rho
sig1=1; %sigma1^2
sig2=1; %sigma2^2
sigma=[sig1 r12*sqrt(sig1*sig2);r12*sqrt(sig1*sig2) sig2];
invs=1/(sig1*sig2-r12^2*sig1*sig2)*[sig2 -r12*sqrt(sig1*sig2); -r12*sqrt(sig1*sig2) sig1];

a=0; %lower bound for design space S
b=500;  %upper bound for design space S

u=zeros(1,N); %initial value for design points
for i=1:N
    u(1,i)=a+(i-1)*(b-a)/(N-1);
end

% r=2 is the number of response model
% q=q1+q2=2+2=4 is the number of parameters in total
U=zeros(2,4,N);
G=zeros(4,4,N);
% construct Uj j=1,...,N.
for i=1:N
    U(1,1:2,i)=[-alpha1*u(i)/(u(i)+beta1)^2 u(i)/(u(i)+beta1) ];
    U(2,3:4,i)=[-alpha2*u(i)/(u(i)+beta2)^2 u(i)/(u(i)+beta2)];
    G(:,:,i)=U(:,:,i)'*invs*U(:,:,i);
end

% cvx
cvx_begin
   cvx_precision high
   variable w(N);
   expression H(4,4);  %target function
   
   %get H
   for i=1:N
    H=H+G(:,:,i)*w(i);
   end
   
   minimize (matrix_frac(my_g',H))
   subject to
           sum(w)==1;
           w>=0;
cvx_end
design=[u([find(w>0.0009)]);w(find(w>0.0009))']

%check for optimality
z=zeros(4,4,N);
A=zeros(4,4);
for i=1:N
    z(:,:,i)=w(i)*G(:,:,i);
    A=A+z(:,:,i);
end
invA=inv(A);
L=my_g';
phi=trace(L'/A*L);
phi_A=zeros(N,1);
check=zeros(N,1);
for i=1:N
    phi_A(i)=trace(sigma\U(:,:,i)/A*L*L'/A*U(:,:,i)');
    check(i)=phi_A(i)-phi;
end
delta=max(check)